
# coding: utf-8

# In[20]:



import sys
from keras.models import Sequential
from keras.layers import Dense, Dropout, Merge,Activation
from keras.layers.embeddings import Embedding
from keras.layers.recurrent import SimpleRNN
from keras.initializers import RandomUniform
from keras.optimizers import Adamax
from keras.regularizers import l2
from itertools import chain
import numpy as np
from collections import defaultdict
from keras.preprocessing import sequence
from itertools import chain
from keras.utils import np_utils
np.random.seed(42)


# In[21]:


def readFile(path):
    """
    Load sentences. A line must contain at least a word and its tag.
    Sentences are separated by empty lines.
    """
    sentences = []
    sentence = []
    for line in open(path):
        line = line.rstrip()
        if not line:
            if len(sentence) > 0:
                sentences.append(sentence)
                sentence = []
        else:
            word = line.split()
            assert len(word) >= 2
            sentence.append(word)
    if len(sentence) > 0:
        sentences.append(sentence)
    return sentences


# In[22]:


train_sentences = readFile("conll2003/en/pos/train.txt")
test_sentences = readFile("conll2003/en/pos/test.txt")
dev_sentences = readFile("conll2003/en/pos/valid.txt")


# In[23]:


w2i = defaultdict(lambda: len(w2i))
t2i = defaultdict(lambda: len(t2i))
PAD=w2i['<padding>']
UNK = w2i["<unk>"]


# In[24]:


for sent in train_sentences:
    for token in sent:
        w2i[token[0]]
w2i=defaultdict(lambda : UNK, w2i) 
i2w = {v: k for k, v in w2i.items()}

n_words=len(w2i)

vocabulary=frozenset([k for k,v in w2i.items()])
MAX_LEN=max([len(word) for word in vocabulary]) #longest token


# In[25]:


for word_label in train_sentences:
    for label in word_label:
        t2i[label[1]]
i2t={v: k for k, v in t2i.items()}
tags= set([k for k,v in t2i.items()])
n_tags=len(t2i)


# In[26]:


transform_train_sentences = [[w2i[token[0]] for token in train_sent] for train_sent in train_sentences]
transform_train_labels = [[t2i[label[1]] for label in train_label] for train_label in train_sentences]
#repeat same for dev dataset
transform_dev_sentences= [[w2i[token[0]] for token in dev_sent] for dev_sent in dev_sentences]
transform_dev_labels=[[t2i[label[1]] for label in dev_label] for dev_label in dev_sentences]

#print (transform_dev_sentences[0])
#print (transform_dev_labels[0])


# In[27]:


def contextwin(l, win):
    '''
    win :: int corresponding to the size of the window
    given a list of indexes composing a sentence

    l :: array containing the word indexes

    it will return a list of list of indexes corresponding
    to context windows surrounding each word in the sentence
    '''
    assert (win % 2) == 1
    assert win >= 1
    l = list(l)

    lpadded = win // 2 * [PAD] + l + win // 2 * [PAD]
    out = [lpadded[i:(i + win)] for i in range(len(l))]
    assert len(out) == len(l)
    return out


# In[28]:


context_window=5
#each training instance will have sequence length of 5. 
#The token will be in the middle surrounded by the  right and left tokens.


# In[29]:


print(transform_train_sentences[1])


# In[30]:


#represent above tweet  tokens with context window of 5
print(contextwin(transform_train_sentences[1],context_window))


# In[31]:


X_train= [ contextwin(train_sent,context_window) for train_sent in transform_train_sentences]


# In[32]:


X_train[:3]


# In[33]:


# we flatten the list
X_train=list(chain(*X_train))


# In[34]:


X_train = np.array(X_train)
X_train.shape


# In[35]:


Y_train=list(chain(*transform_train_labels))
Y_train=np_utils.to_categorical(np.array(Y_train)) # one hot encoding


# In[36]:


Y_train[:1]


# In[37]:


assert len(X_train)==len(Y_train)


# In[38]:


batch_size=128
nb_epoch=5
word_model=Sequential()
word_model.add(Embedding(output_dim=300, input_dim=n_words,input_length=context_window,mask_zero=True))
word_model.add(SimpleRNN(100, return_sequences=False))
word_model.summary()
word_model.add(Dense(n_tags))
word_model.add(Activation('softmax'))


# In[39]:


word_model.compile(loss='categorical_crossentropy', optimizer="adam", metrics=['accuracy'])

word_model.fit(X_train, Y_train,
                batch_size=batch_size,
                epochs=nb_epoch,
                verbose=1,
                shuffle=True
                )

